<?php
namespace Vitrium\Drm\HttpClient;

use \Vitrium\Drm\VitriumClasses\ChallengeContext;
use \Vitrium\Drm\VitriumClasses\DocPostContext;
use \Vitrium\Drm\VitriumClasses\DocumentVersion;
use \Vitrium\Drm\VitriumClasses\Folder;
use \Vitrium\Drm\VitriumClasses\ResponseContext;
use \Vitrium\Drm\VitriumClasses\VitriumApiKeys;

class VitriumClient extends HttpClient {
    // Session and account tokens required for Vitrium API access
    private $vitriumKeys;

    /**
    * Public constructor
    *
    * @param array  $options  cUrl options
    * @param Logger $logger   Dependency injected instance of \Vitrium\Drm\Logger\Logger
    * @param Helper $helper   Dependency injected instance of \Vitrium\Drm\Helper\Data
    */
    public function __construct($options, \Vitrium\Drm\Logger\Logger $logger, \Vitrium\Drm\Helper\Data $helper) {
        parent::__construct($options, $logger, $helper);
        $this->login();

        $this->logger->debug('Vitrium Client initialized; API Keys: ' . (!isset($this->vitriumKeys) ? 'not set' : $this->vitriumKeys->LogFormat()));
    }    

    /**
    * Creates and returns default request headers
    *
    * @param  mixed $body Request payload
    * @return array $headers Array of headers composed of base HttpClient headers plus Vitrium API key headers, if configured.
    */
    protected function getRequestHeaders($body) {
        $headers = parent::getRequestHeaders($body);
        if (isset($this->vitriumKeys) && $this->vitriumKeys->AreKeysSet()) {
            $headers['X-VITR-SESSION-TOKEN'] = $this->vitriumKeys->getSessionKey();
            $headers['X-VITR-ACCOUNT-TOKEN'] = $this->vitriumKeys->getAccountKey();
        }
        return $headers;
    }

    /**
    * Performs Vitrium login and sets Vitrium API keys properties
    * @return void
    */
    public function login() {
        // Invalidate any existing Vitrium keys
        $this->vitriumKeys = null;

        try {
            $challengeContext = new ChallengeContext($this->helper);
            $clientNonce = $challengeContext->getClientNonce();
            $challengeResult = $this->post('login/challenge', $challengeContext);

            $username = $this->config['vitrium']['username'];
            $password = $this->config['vitrium']['password'];

            $responseContext = new ResponseContext();
            $responseContext->setClientNonce($clientNonce);
            $responseContext->setUsername($username);
            $responseContext->setApplicationId('Vitrium Magento Extension');
            $responseContext->setClientHash(strtoupper(hash_hmac('sha1', strtolower($clientNonce) . strtolower($challengeResult->ServerNonce) . $password, $password)));

            $responseResult = null;
            try {
                $responseResult = $this->post('login/response', $responseContext);
            }
            catch (HttpClientException $e) {            
                $this->logger->err('HTTP Error accessing ' . $e->getRequest()->getUrl() . ': ' . $e->getMessage() . ' [' . $e->getCode() . ']');
            }        

            if (!isset($responseResult) || !property_exists($responseResult, 'ApiSession') || !property_exists($responseResult, 'Accounts'))
                return;

            $this->vitriumKeys = new VitriumApiKeys($responseResult->ApiSession->Token, $responseResult->Accounts[0]->Id);
        }
        catch (Exception $ex) {
            $this->logger->err('Exception in Vitrium login: ' . $ex);
        }
    }

    /**
    * Retrieves or creates a folder by the given name with the given parent folder ID in Vitrium
    *
    * @param string $name Folder name to search for in Vitrium
    * @param GUID   $parentId  Vitrium Folder ID of a parent folder; can be null
    * @return Folder $folder 
    */
    public function getOrCreateFolder($name, $parentId = null) {
        $searchResults = $this->get('folder?filter=[{"Name":"' . $name .'"}]');

        if ($searchResults->TotalRecords == 0 || empty($searchResults->Results)) {   // Folder was not found; let's create it
            $newFolder = new Folder();
            $newFolder->setId($this->helper->createGuid());
            $newFolder->setName($name);
            $newFolder->setParenFolderId($parentId);

            $folder = $this->post('folder', $newFolder);
        }
        else {
            // TODO: Instead of returning the first folder found by name, try to find the one with the proper parent folder if $parentId is supplied
            $folder = new Folder($searchResults->Results[0]);
        }

        return $folder;
    }

    /**
    * Retrieves content setting ID by name
    *
    * @param string $name Content Setting name to search for in Vitrium
    * @return Vitrium Content Setting GUID ID as string, or null if not found.
    */
    public function getContentSettingIdByName($name) {
        $searchResults = $this->get('policy?filter=[{"Name":"' . $name .'"}]');
        return ($searchResults->TotalRecords == 0 || empty($searchResults->Results) ? null : $searchResults->Results[0]->Id);
    }

    /**
    * Uploads and protects content.
    *
    * @param string $filepath Full path on the local file storage to the content to be uploaded.
    * @param GUID $folderId Vitrium Folder ID. Optional; if ommitted, content is uploaded to default Main Folder.
    * @param GUID $contentSettingId  Vitrium Content Setting ID. Optional; if ommitted, content is uploaded with no Content Settings set.
    * @return Version containing the created Vitrium Document Version information, or null if the content protection failed.
    */
    public function protectContent($filepath, $folderId = null, $contentSettingId = null) {
        try {
            $filename = pathinfo($filepath, PATHINFO_BASENAME);

            $fileDataPart = array();
            $fileDataPart['headers'] = [ 
                'Content-Disposition' => 'filename="' . $filename . '"',
                'Content-Type' => 'application/pdf'
            ];
            $fileDataPart['data'] = file_get_contents($filepath);

            $docSettings = new DocPostContext();
            if (!empty($folderId))
                $docSettings->setFolderId($folderId);
            if (!empty($contentSettingId))
                $docSettings->setPolicyId($contentSettingId);
            $docSettings->setFileName($filename);
            $docSettings->setTitle($filename);
            $docSettings->setSkipPages("");
            $docSettings->setExternalKey("");
            $docSettings->setVersionName("Version: 1");
            $docSettings->setEnableVersionNotification(true);

            $settingsDataPart = array();
            $settingsDataPart['headers'] = [ 
                'Content-Type' => 'application/json'
            ];
            $settingsDataPart['data'] = $docSettings;

            // Note that this will be a multi-part post
            $postResult = $this->post('doc', [ 'rawfile' => $fileDataPart, 'settings' => $settingsDataPart ]);
            return new DocumentVersion($postResult);
        }
        catch (Exception $ex) {
            $this->logger->err('Exception in Vitrium Client protectContent: ' . $ex);
            return null;
        }
    }
}
